Package: CENTREannotation
Title: Hub package for the annotation data of CENTRE (GENCODE v40 and
        SCREEN v3)
Version: 0.99.1
Authors@R: c(person(given = "Sara", family = "Lopez Ruiz de Vargas",
    email = "lopez_s@molgen.mpg.de",
	  role = c("aut", "cre"),
		comment = c(ORCID = "0000-0002-1290-4774")))
Description: This is an AnnotationHub package for the CENTRE Bioconductor 
    software package. It contains the GENCODE version 40 annotation and ENCODE 
    Registry of candidate cis-regulatory elements (cCREs) version 3. All 
    for Human hg38 genome.
License: Artistic-2.0
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
biocViews: AnnotationData, AnnotationHub
Depends: R (>= 4.5.0)
Imports: AnnotationHub, RSQLite, DBI, BiocGenerics, methods
VignetteBuilder: knitr
Suggests: knitr, rmarkdown, testthat, BiocStyle
URL: https://github.com/slrvv/CENTREannotation
BugReports: https://github.com/slrvv/CENTREannotation/issues
git_url: https://git.bioconductor.org/packages/CENTREannotation
git_branch: devel
git_last_commit: 6397f51
git_last_commit_date: 2025-08-04
Repository: Bioconductor 3.22
Date/Publication: 2025-08-06
NeedsCompilation: no
Packaged: 2025-08-06 10:31:45 UTC; biocbuild
Author: Sara Lopez Ruiz de Vargas [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1290-4774>)
Maintainer: Sara Lopez Ruiz de Vargas <lopez_s@molgen.mpg.de>

# Generated by roxygen2: do not edit by hand

export(fetch_data)
export(tables)
exportClasses(CENTREannotDb)
import(AnnotationHub)
import(BiocGenerics)
importClassesFrom(DBI,DBIConnection)
importFrom(DBI,dbConnect)
importFrom(DBI,dbDisconnect)
importFrom(DBI,dbDriver)
importFrom(DBI,dbGetQuery)
importFrom(DBI,dbIsValid)
importFrom(DBI,dbListTables)
importFrom(RSQLite,SQLITE_RO)
importFrom(methods,.hasSlot)
importFrom(methods,is)
importFrom(methods,new)

## The set up of the class and properties is based on the CompDb class and
## package

#' @name CENTREannotDb
#'
#' @import BiocGenerics AnnotationHub
#' @title Class object for the CENTRE annotation data
#'
#' @aliases CENTREannotDb-class 
#' @return class CENTREannotDb
#'
#' @description
#'
#' The CENTRE annotation data is accessed through objects of `CENTREannotDb` class.
#' That either provides access to the ENCODE screen V3 annotation
#' for Human https://screen.encodeproject.org or gives access to
#' GENCODE basic gene annotation version 40.
#'
#' @details
#'
#' Using `tables(x)` on a `CENTREannotDb` object returns the tables and columns
#' for each table in the database.
#'
#' @examples
#' ##load the screen V3 annotation
#' ah <- AnnotationHub::AnnotationHub()
#' screen <- ah[["AH116731"]]
#' tables(screen) #get all tables and columns in the data base
#' @references Based on CompoundDb::CompDb.

#' @importClassesFrom DBI DBIConnection
setClassUnion("DBIConnectionOrNULL", c("DBIConnection", "NULL"))

#' @importFrom methods new is
#'
#' @exportClass CENTREannotDb
.CENTREannotDb <- setClass("CENTREannotDb",
    slots = c(
        conn = "DBIConnectionOrNULL",
        .properties = "list",
        dbname = "character",
        dbflags = "integer",
        packageName = "character"
    ),
    prototype = list(
        .properties = list(),
        conn = NULL,
        dbname = character(),
        dbflags = 1L,
        packageName = character()
    )
)


#' @param x sqlite file path
#
#' @importFrom RSQLite SQLITE_RO
#'
#' @rdname CENTREannotDb
CENTREannotDb <- function(x) {
    return(.initialize_centredb(.CENTREannotDb(
        conn = x,
        dbflags = SQLITE_RO,
        packageName = "CENTREannotation"
    )))
}

#' @importFrom DBI dbDriver dbGetQuery dbConnect dbListTables dbDisconnect dbIsValid
#' 
.initialize_centredb <- function(x) {
    con <- .dbconn(x)
    x@dbname <- dbfile(con)
    if (length(.dbname(x)) && !is.null(con)) {
        on.exit(dbDisconnect(con))
    }
    ## fetch all tables and all columns for all tables.
    tbl_nms <- dbListTables(con)
    tbls <- lapply(tbl_nms, function(z) {
        colnames(dbGetQuery(con, paste0("select * from ", z, " limit 1")))
    })
    names(tbls) <- tbl_nms
    x@.properties$tables <- tbls
    x
}
.metadata <- function(x) {
    if (!is(x, "DBIConnection")) {
        n <- .dbname(x)
        x <- .dbconn(x)
        if (length(n) && !is.null(x)) {
            on.exit(dbDisconnect(x))
        }
    }
    dbGetQuery(x, "select * from metadata")
}

.metadata_value <- function(x, key) {
    metad <- .metadata(x)
    metad[metad$name == key, "value"]
}

#' @importFrom methods .hasSlot
.dbflags <- function(x) {
    if (.hasSlot(x, "dbflags")) {
        x@dbflags
    } else {
        1L
    }
}

#' @importFrom DBI dbIsValid
.dbconn <- function(x) {
    if (!dbIsValid(x@conn)) {
        dbConnect(dbDriver("SQLite"), dbname = dbfile(x@conn), flags = .dbflags(x))
    } else {
        x@conn
    }
}

.dbname <- function(x) {
    if (.hasSlot(x, "dbname")) {
        x@dbname
    } else {
        character()
    }
}


#' @description Get a list of all tables and their columns.
#'
#' @param x `CENTREannotDb` object.
#'
#' @param name optional `character` to return the table/columns for specified
#'     tables.
#'
#' @param metadata `logical(1)` whether the metadata should be returned too.
#'
#' @noRd
.tables <- function(x, name, metadata = FALSE) {
    tbls <- .get_property(x, "tables")
    if (!missing(name)) {
        tbls <- tbls[name]
    }
    if (!metadata) {
        tbls <- tbls[names(tbls) != "metadata"]
    }
    tbls
}

#' @export
#'
#' @rdname CENTREannotDb
tables <- function(x) {
    con <- .dbconn(x)
    x <- CENTREannotDb(con)
    .tables(x)
}
.get_property <- function(x, name) {
    x@.properties[[name]]
}

test_that("CENTREannotDb obj connection works as intented", {
    ah <- AnnotationHub::AnnotationHub()
    CENTREannotgeneDb <- ah[["AH116730"]]
    CENTREannotgeneDb <- CENTREannotDb(CENTREannotgeneDb@conn)
    expect_false(RSQLite::dbIsValid(CENTREannotgeneDb@conn))
    expect_true(RSQLite::dbIsValid(.dbconn(CENTREannotgeneDb)))
    
    
    ah <- AnnotationHub::AnnotationHub()
    CENTREannotenhDb <- ah[["AH116731"]]
    CENTREannotenhDb <- CENTREannotDb(CENTREannotenhDb@conn)
    expect_false(RSQLite::dbIsValid(CENTREannotenhDb@conn))
    expect_true(RSQLite::dbIsValid(.dbconn(CENTREannotenhDb)))
})

#' @name CENTREannotation-package
#'
#' @title CENTREannotation: Hub package for the annotation data of CENTRE
#' (GENCODE v40 and SCREEN v3)
#'
#' @format SQLite databases with GENCODE v40 and SCREEN v3 data.
#'
#' @source \url{https://www.gencodegenes.org/human/release_40.html}
#' \url{https://screen.encodeproject.org}
#'
#' @description
#' This is an AnnotationHub package for the CENTRE Bioconductor
#' software package. It contains the GENCODE version 40 annotation and ENCODE
#' Registry of candidate cis-regulatory elements (cCREs) version 3. All
# 'for Human hg38 genome.
#'
#' @examples
#' \donttest{
#' library(AnnotationHub)
#' hub <- AnnotationHub()
#' eh <- query(hub, "CENTREannotation")
#' }
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

##########################################################################
# CENTREannotation: GENCODE and ENCODE cCREs
#########################################################################

# The data for GENCODE comprehensive gene annotation on the reference
# chromosomes was downloaded from
# https://www.gencodegenes.org/human/release_40.html
# in GTF file format Release 40. The file was then transformed from GTF to bed
# and the relevant columns where selected (chr, start, end, gene_id,
# orientation, transcript_id, gene_name, description, transcription_start) and
# we excluded exon annotation. The transcription start site was extended to a
# size of 499 for all genes. We include the column gene_id1 which contains the
# gene ID without version identifier.
# The data was transformed into a SQLite database using RSQLite


# The data for ENCODE cCREs was downloaded from https://screen.encodeproject.org V3
# under Downloads for all types of regulatory elements in Human.
# The BED file was downloaded and we added names to the columns. The regulatory
# element region was extended to a have a size of 500 and the middle point in the
# new reg element range was computed.
# The data was transformed into an SQLite database using RSQLite

##########################################################################
##
## CENTREannotation: GENCODE v40, ENCODE cCREs v3 and HeLa experimental
## data for the example.
##
##########################################################################

meta <- data.frame(
	Title = c("GENCODE basic gene annotation v40",
	"ENCODE Registry of cCREs V3"),
	Description = c(paste0("GENCODE comprehensive basic gene annotation v40"
	," on reference chromosomes only"),
	"ENCODE cCREs V3 all human cCREs"
	),
	BiocVersion = c(rep("3.19", 2)),
	Genome = rep("GRCh38", 2), 
	SourceType = c("GTF", "BED"),
	SourceUrl = c("https://www.gencodegenes.org/human/release_40.html",
	"https://screen.encodeproject.org"),
	SourceVersion = c("V40", "V3"),
	Species = rep("Homo sapiens", 2),
	TaxonomyId = rep(9606, 2),
	Coordinate_1_based = TRUE, 
	DataProvider = c("GENCODE", "ENCODE cCREs"),
	Maintainer = "Sara Lopez <lopez_s@molgen.mpg.de>",
	RDataClass = c("SQLiteConnection","SQLiteConnection"),
	DispatchClass = c(rep("SQLiteFile", 2)),
	Location_Prefix = c(rep("http://owww.molgen.mpg.de/~CENTRE_data/", 2)),
	RDataPath = c("CENTREannotationData/gencode_hg38_v40.db", 
	              "CENTREannotationData/ENCODEAnnotation-GRCh38-cCREsV3.db"),
	Tags = c(rep("AnnotationHub:AnnotationData:Organism", 2))
)

write.csv(meta, "inst/extdata/metadata.csv", row.names=FALSE)
 	

## A simple query engine for the CENTREannotation package based on the
## CompDb package adapted to CENTREannotation

#' @description
#'
#' Utility function to create a SQL query for a `CENTREannotDb` database
#' given the provided column names, entries and column filter.
#'
#' @param x `CENTREannotDb` type object
#' @param columns Columns to select. Equivalent to X in SELECT X
#' @param entries Element ID to select. Equivalent to ID in SELECT X FROM TABLE
#' WHERE ID in Z.
#' @param column_filter Column on which to apply filter. Equivalent to Z in
#' SELECT X FROM TABLE WHERE ID in Z.
#' @details
#'
#' + Check first that the `columns` are valid
#' + Fetch the data based on entries and column on which to filter.
#'
#' @md
#' @references Based on the internal query engine of [CompoundDb]
#' @noRd
.build_query <- function(x, columns, entries, column_filter) {
    if (missing(x)) {
        stop(" 'x' is required")
    }
    if (missing(columns)) {
        stop("'columns' is required")
    }

    tbls <- .tables(x)
    col_m <- columns[!columns %in% unique(unlist(tbls, use.names = FALSE))]
    msg <- paste(col_m, collapse = ", ")
    if (length(col_m != 0)) {
        stop(
            "Columns ", msg,
            " are not present in the database. Use 'tables' to list ",
            "all tables and their columns."
        )
    }
    paste0(
        paste0("select ", paste0(columns, collapse = ",")),
        paste0(" from ", names(tbls)),
        .where(entries, column_filter)
    )
}

#' @description
#'
#' Create the *where* condition for the SQL query based on the provided
#' filter column
#'
#' @md
#'
#' @noRd
.where <- function(entries, column_filter) {
    if (!missing(column_filter)) {
        paste0(
            " where ", column_filter, " in (",
            paste0(sprintf("'%s'", entries), collapse = ", "), ")"
        )
    }
}

#' @name fetch_data
#'
#' @import BiocGenerics
#' @title Fetch data from the CENTREannotDb databases
#'
#' @description
#' Main interface to fetch data from the CENTREannotation package databases
#' through the `CENTREannotDb` objects.
#'
#' @param x  A `CENTREannotDb` object.
#' @param columns Columns to select (vector or string). Equivalent to X in
#' SELECT X.
#' @param entries Element ID to select (vector or sting). Equivalent to ID in
#' SELECT X FROM TABLE WHERE ID in Z. If entries or column_filter is missing
#' the program assumes the query is SELECT X FROM TABLE.
#' @param column_filter Column on which to apply filter. Equivalent to Z in
#' SELECT X FROM TABLE WHERE ID in Z. If entries or column_filter is missing the
#' program assumes the query is SELECT X FROM TABLE.
#'
#' @return data.frame with the queried data.
#'
#' @references Based on the internal query engine of `CompoundDb`
#'
#' @examples
#' ah <- AnnotationHub::AnnotationHub()
#' CENTREannotenhDb <- ah[["AH116731"]]
#' res <- fetch_data(CENTREannotenhDb,
#'     columns = c("enhancer_id", "start"),
#'     entries = c("EH38E1519134", "EH38E1519132"),
#'     column_filter = "enhancer_id"
#' )
#' @export
fetch_data <- function(x, columns, entries, column_filter) {
    
    x <- CENTREannotDb(x@conn)
    con <- .dbconn(x) #connect to database to retrieve records
    if (length(.dbname(x)) && !is.null(con)) {
        on.exit(dbDisconnect(con))
    }
    data <- dbGetQuery(con, .build_query(x,
        columns = columns,
        entries = entries,
        column_filter = column_filter
    ))
    return(data)
}

test_that("fetch_data works as intented", {
    expected <- data.frame(
        enhancer_id = c("EH38E1519132", "EH38E1519134"),
        start = c(9363676, 9364325)
    )
    ah <- AnnotationHub::AnnotationHub()
    CENTREannotenhDb <- ah[["AH116731"]]
    
    fetched <- fetch_data(CENTREannotenhDb,
        columns = c("enhancer_id", "start"),
        entries = c("EH38E1519134", "EH38E1519132"),
        column_filter = "enhancer_id"
    )
    expect_equal(fetched, expected, tolerance = 1e-6)

    ah <- AnnotationHub::AnnotationHub()
    CENTREannotgeneDb <- ah[["AH116730"]]
    fetched <- fetch_data(CENTREannotgeneDb,
        columns = c("gene_id1", "chr"),
        entries = c("ENSG00000000419"),
        column_filter = "gene_id1"
    )

    expected <- data.frame(
        gene_id1 = c("ENSG00000000419"),
        chr = c("chr20")
    )

    expect_equal(fetched, expected, tolerance = 1e-6)
    # catch error when column name doesnt exist
    expect_error(fetch_data(CENTREannotenhDb,
        columns = c("enhancer_id", "start", "blipblup"),
        entries = c("EH38E1519134", "EH38E1519132"),
        column_filter = "enhancer_id"
    ))
    # catch error missing Db oject
    expect_error(fetch_data(
        columns = c("enhancer_id", "start"),
        entries = c("EH38E1519134", "EH38E1519132"),
        column_filter = "enhancer_id"
    ))

    # catch error missing columns
    expect_error(fetch_data(CENTREannotenhDb,
        entries = c("EH38E1519134", "EH38E1519132"),
        column_filter = "enhancer_id"
    ))
})

---
title: "CENTREannotation"
author: "Sara Lopez Ruiz de Vargas"
date: "`r doc_date()`"
vignette: >
  %\VignetteIndexEntry{CENTRE AnnotationHub datasets}
  %\VignetteEngine{knitr::rmarkdown}
output: 
  BiocStyle::html_document
---

# CENTREannotation: An AnnotationHub package with the ENCODE cCREs V3 and 
GENCODE basic gene annotation v40 for the CENTRE package

**CENTRE** is a package for Cell-type specific ENhancer Target pREdiction, that 
follows this workflow: 

`createPairs()` -> `computeGenericFeatures()` -> `computeCellTypeFeatures()` 
-> `centreClassification()`

The step `CENTRE::createPairs()` creates all possible enhancer-gene pairs at 
500kb of the input genes or enhancers. For this step CENTRE uses the 
[ENCODE SCREEN v3 enhancer annotation](https://screen.encodeproject.org) and the
[GENCODE v40 gene annotation](https://www.gencodegenes.org/human/release_40.html)


All of the data in the CENTREannotation package can be accessed through 
AnnotationHub:

```{r load AnnotationHub, results='hide'}
library(AnnotationHub, quietly = TRUE)
```

```{r setup}
hub <- AnnotationHub()
ah <- query(hub, "CENTREannotation")
ah
```

The GENCODE database can be accessed using `ah[["AH116730"]]` that returns a 
CENTREannotdb object. The SCREEN database can be accessed using `ah[["AH116731"]]`
which also returns a CENTREannotdb object.

## How to use the CENTREannotDb objects.

The objects  CENTREannotgeneDb and CENTREannotenhDb  represent the GENCODE
and ENCODE databases respectively
```{r CENTREannotDb}
library(CENTREannotation)
CENTREannotgeneDb <- ah[["AH116730"]]
CENTREannotenhDb <- ah[["AH116731"]]
```

The database can be used as follows:

- `tables()`: shows all the tables and columns in the database
- `fetch_data()`: function to select data from the database (see man pages) 

```{r use CENTREannotDb}
tables(CENTREannotenhDb)
res <- fetch_data(CENTREannotenhDb,
    columns = c("enhancer_id", "start"),
    entries = c("EH38E1519134", "EH38E1519132"),
    column_filter = "enhancer_id"
)
```

```{r sessionInfo}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CENTREannotation-package.R
\docType{package}
\name{CENTREannotation-package}
\alias{CENTREannotation}
\alias{CENTREannotation-package}
\title{CENTREannotation: Hub package for the annotation data of CENTRE
(GENCODE v40 and SCREEN v3)}
\format{
SQLite databases with GENCODE v40 and SCREEN v3 data.
}
\source{
\url{https://www.gencodegenes.org/human/release_40.html}
\url{https://screen.encodeproject.org}
}
\description{
This is an AnnotationHub package for the CENTRE Bioconductor
software package. It contains the GENCODE version 40 annotation and ENCODE
Registry of candidate cis-regulatory elements (cCREs) version 3. All
}
\examples{
\donttest{
library(AnnotationHub)
hub <- AnnotationHub()
eh <- query(hub, "CENTREannotation")
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/slrvv/CENTREannotation}
  \item Report bugs at \url{https://github.com/slrvv/CENTREannotation/issues}
}

}
\author{
\strong{Maintainer}: Sara Lopez Ruiz de Vargas \email{lopez_s@molgen.mpg.de} (\href{https://orcid.org/0000-0002-1290-4774}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CENTREannotDb.R
\docType{class}
\name{CENTREannotDb}
\alias{CENTREannotDb}
\alias{CENTREannotDb-class}
\alias{tables}
\title{Class object for the CENTRE annotation data}
\usage{
CENTREannotDb(x)

tables(x)
}
\arguments{
\item{x}{sqlite file path}
}
\value{
class CENTREannotDb
}
\description{
The CENTRE annotation data is accessed through objects of \code{CENTREannotDb} class.
That either provides access to the ENCODE screen V3 annotation
for Human https://screen.encodeproject.org or gives access to
GENCODE basic gene annotation version 40.
}
\details{
Using \code{tables(x)} on a \code{CENTREannotDb} object returns the tables and columns
for each table in the database.
}
\examples{
##load the screen V3 annotation
ah <- AnnotationHub::AnnotationHub()
screen <- ah[["AH116731"]]
tables(screen) #get all tables and columns in the data base
}
\references{
Based on CompoundDb::CompDb.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/queryEngine.R
\name{fetch_data}
\alias{fetch_data}
\title{Fetch data from the CENTREannotDb databases}
\usage{
fetch_data(x, columns, entries, column_filter)
}
\arguments{
\item{x}{A \code{CENTREannotDb} object.}

\item{columns}{Columns to select (vector or string). Equivalent to X in
SELECT X.}

\item{entries}{Element ID to select (vector or sting). Equivalent to ID in
SELECT X FROM TABLE WHERE ID in Z. If entries or column_filter is missing
the program assumes the query is SELECT X FROM TABLE.}

\item{column_filter}{Column on which to apply filter. Equivalent to Z in
SELECT X FROM TABLE WHERE ID in Z. If entries or column_filter is missing the
program assumes the query is SELECT X FROM TABLE.}
}
\value{
data.frame with the queried data.
}
\description{
Main interface to fetch data from the CENTREannotation package databases
through the \code{CENTREannotDb} objects.
}
\examples{
ah <- AnnotationHub::AnnotationHub()
CENTREannotenhDb <- ah[["AH116731"]]
res <- fetch_data(CENTREannotenhDb,
    columns = c("enhancer_id", "start"),
    entries = c("EH38E1519134", "EH38E1519132"),
    column_filter = "enhancer_id"
)
}
\references{
Based on the internal query engine of \code{CompoundDb}
}
